@@ -1,9 +1,10 @@
-//! Operator implementations on `BitArray`.
+//! Port of the `[T; N]` operator implementations.
 
 use crate::{
 	array::BitArray,
 	order::BitOrder,
 	slice::BitSlice,
+	store::BitStore,
 	view::BitView,
 };
 
@@ -21,27 +22,25 @@ use core::ops::{
 	Not,
 };
 
-#[cfg(not(tarpaulin_include))]
 impl<O, V, Rhs> BitAnd<Rhs> for BitArray<O, V>
 where
 	O: BitOrder,
-	V: BitView + Sized,
+	V: BitView,
 	BitSlice<O, V::Store>: BitAndAssign<Rhs>,
 {
 	type Output = Self;
 
 	#[inline]
 	fn bitand(mut self, rhs: Rhs) -> Self::Output {
-		*self.as_mut_bitslice() &= rhs;
+		self &= rhs;
 		self
 	}
 }
 
-#[cfg(not(tarpaulin_include))]
 impl<O, V, Rhs> BitAndAssign<Rhs> for BitArray<O, V>
 where
 	O: BitOrder,
-	V: BitView + Sized,
+	V: BitView,
 	BitSlice<O, V::Store>: BitAndAssign<Rhs>,
 {
 	#[inline]
@@ -50,27 +49,25 @@ where
 	}
 }
 
-#[cfg(not(tarpaulin_include))]
 impl<O, V, Rhs> BitOr<Rhs> for BitArray<O, V>
 where
 	O: BitOrder,
-	V: BitView + Sized,
+	V: BitView,
 	BitSlice<O, V::Store>: BitOrAssign<Rhs>,
 {
 	type Output = Self;
 
 	#[inline]
 	fn bitor(mut self, rhs: Rhs) -> Self::Output {
-		*self.as_mut_bitslice() |= rhs;
+		self |= rhs;
 		self
 	}
 }
 
-#[cfg(not(tarpaulin_include))]
 impl<O, V, Rhs> BitOrAssign<Rhs> for BitArray<O, V>
 where
 	O: BitOrder,
-	V: BitView + Sized,
+	V: BitView,
 	BitSlice<O, V::Store>: BitOrAssign<Rhs>,
 {
 	#[inline]
@@ -79,27 +76,25 @@ where
 	}
 }
 
-#[cfg(not(tarpaulin_include))]
 impl<O, V, Rhs> BitXor<Rhs> for BitArray<O, V>
 where
 	O: BitOrder,
-	V: BitView + Sized,
+	V: BitView,
 	BitSlice<O, V::Store>: BitXorAssign<Rhs>,
 {
 	type Output = Self;
 
 	#[inline]
 	fn bitxor(mut self, rhs: Rhs) -> Self::Output {
-		*self.as_mut_bitslice() ^= rhs;
+		self ^= rhs;
 		self
 	}
 }
 
-#[cfg(not(tarpaulin_include))]
 impl<O, V, Rhs> BitXorAssign<Rhs> for BitArray<O, V>
 where
 	O: BitOrder,
-	V: BitView + Sized,
+	V: BitView,
 	BitSlice<O, V::Store>: BitXorAssign<Rhs>,
 {
 	#[inline]
@@ -112,7 +107,7 @@ where
 impl<O, V> Deref for BitArray<O, V>
 where
 	O: BitOrder,
-	V: BitView + Sized,
+	V: BitView,
 {
 	type Target = BitSlice<O, V::Store>;
 
@@ -126,7 +121,7 @@ where
 impl<O, V> DerefMut for BitArray<O, V>
 where
 	O: BitOrder,
-	V: BitView + Sized,
+	V: BitView,
 {
 	#[inline(always)]
 	fn deref_mut(&mut self) -> &mut Self::Target {
@@ -137,7 +132,7 @@ where
 impl<O, V, Idx> Index<Idx> for BitArray<O, V>
 where
 	O: BitOrder,
-	V: BitView + Sized,
+	V: BitView,
 	BitSlice<O, V::Store>: Index<Idx>,
 {
 	type Output = <BitSlice<O, V::Store> as Index<Idx>>::Output;
@@ -151,7 +146,7 @@ where
 impl<O, V, Idx> IndexMut<Idx> for BitArray<O, V>
 where
 	O: BitOrder,
-	V: BitView + Sized,
+	V: BitView,
 	BitSlice<O, V::Store>: IndexMut<Idx>,
 {
 	#[inline]
@@ -160,17 +155,18 @@ where
 	}
 }
 
-#[cfg(not(tarpaulin_include))]
 impl<O, V> Not for BitArray<O, V>
 where
 	O: BitOrder,
-	V: BitView + Sized,
+	V: BitView,
 {
 	type Output = Self;
 
 	#[inline]
 	fn not(mut self) -> Self::Output {
-		let _ = !self.as_mut_bitslice();
+		for elem in self.as_mut_slice() {
+			elem.store_value(!elem.load_value());
+		}
 		self
 	}
 }
