@@ -1,23 +1,30 @@
-/*! `bitvec` Prelude
+/*! [`bitvec`] symbol export.
 
-This collects the general public API into a single spot for inclusion, as
+This module collects the general public API into a single spot for inclusion, as
 `use bitvec::prelude::*;`, without polluting the root namespace of the crate.
+
+[`bitvec`]: crate
 !*/
 
 pub use crate::{
+	array::BitArray,
+	bitarr,
 	bits,
-	fields::BitField,
+	field::BitField,
 	order::{
 		BitOrder,
-		Local,
+		LocalBits,
 		Lsb0,
 		Msb0,
 	},
-	slice::{
-		AsBits,
-		BitSlice,
+	ptr::{
+		BitPtr,
+		BitPtrRange,
+		BitRef,
 	},
+	slice::BitSlice,
 	store::BitStore,
+	view::BitView,
 };
 
 #[cfg(feature = "alloc")]
@@ -27,6 +34,3 @@ pub use crate::{
 	boxed::BitBox,
 	vec::BitVec,
 };
-
-#[allow(deprecated)]
-pub use crate::cursor::*;
