@@ -1,35 +1,46 @@
-/*! `serde`-powered de/serialization.
+/*! [`serde`]-powered de/serialization.
 
-This module implements the Serde traits for the `bitvec` types.
+This module implements the Serde traits for the [`bitvec`] types.
 
-`BitSlice` is able to implement `Serialize`, but `serde` does not provide a
-mechanism for deserializing into data borrowed from the calling context. Thus,
-`BitSlice` can only deserialize into `BitBox` or `BitVec`, when built with the
-`"alloc"` feature.
+As [`BitArray`] does not use dynamic indexing for its starting index or its
+length, it implements [`Deserialize`] and [`Serialize`] by forwarding to the
+interior buffer, and adds no additional information. This also renders it
+incapable of deserializing a serialized [`BitSlice`]; it can only deserialize
+bare value sequences.
 
-`BitBox` and `BitVec` implement serialization through `BitSlice`, and
-deserialize `BitSlice`s into themselves.
+[`BitSlice`] is able to implement [`Serialize`], but [`serde`] does not provide
+a behavior to deserialize data into a buffer provided by the calling context, so
+it cannot deserialize into any of the owning structures.
 
-`BitArray` has different behavior: because it always spans the full memory
-buffer, and has no partial edge elements, it de/serializes the underlying memory
-array without any additional information. It is currently incapable of
-deserializing the stream produced by serializing `BitSlice`, and can only
-deserialize the streams produced by `BitArray`s and ordinary arrays.
+[`BitBox`] and [`BitVec`] implement [`Serialize`] through [`BitSlice`], and can
+deserialize the [`BitSlice`] format into themselves.
 
-If you require de/serialization compatibility between `BitArray` and the other
+If you require de/serialization compatibility between [`BitArray`] and the other
 structures, please file an issue.
+
+[`BitArray`]: crate::array::BitArray
+[`BitBox`]: crate::boxed::BitBox
+[`BitSlice`]: crate::slice::BitSlice
+[`BitVec`]: crate::vec::BitVec
+[`Deserialize`]: serde::de::Deserialize
+[`Serialize`]: serde::ser::Serialize
+[`bitvec`]: crate
+[`serde`]: serde
 !*/
 
 #![cfg(feature = "serde")]
 
 use crate::{
 	array::BitArray,
-	devel as dvl,
 	domain::Domain,
-	index::BitRegister,
 	mem::BitMemory,
 	order::BitOrder,
-	pointer::BitPtr,
+	ptr::{
+		AddressError,
+		BitPtr,
+		BitPtrError,
+		BitSpanError,
+	},
 	slice::BitSlice,
 	store::BitStore,
 	view::BitView,
@@ -37,14 +48,12 @@ use crate::{
 
 use core::{
 	cmp,
-	convert::TryInto,
 	fmt::{
 		self,
 		Formatter,
 	},
 	marker::PhantomData,
 	mem::ManuallyDrop,
-	ptr,
 };
 
 use serde::{
@@ -81,8 +90,8 @@ where
 {
 	fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
 	where S: Serializer {
-		let head = self.bitptr().head();
-		let mut state = serializer.serialize_struct("BitSet", 3)?;
+		let head = self.as_bitspan().head();
+		let mut state = serializer.serialize_struct("BitSeq", 3)?;
 
 		state.serialize_field("head", &head.value())?;
 		state.serialize_field("bits", &(self.len() as u64))?;
@@ -111,16 +120,20 @@ impl<O, V> Serialize for BitArray<O, V>
 where
 	O: BitOrder,
 	V: BitView,
-	V::Mem: Serialize,
+	<V::Store as BitStore>::Mem: Serialize,
 {
 	fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
 	where S: Serializer {
-		self.as_raw_slice().serialize(serializer)
+		let ary = self.as_slice();
+		let mut state = serializer.serialize_seq(Some(ary.len()))?;
+		for elem in ary.iter().map(BitStore::load_value) {
+			state.serialize_element(&elem)?;
+		}
+		state.end()
 	}
 }
 
 #[cfg(feature = "alloc")]
-#[cfg(not(tarpaulin_include))]
 impl<O, T> Serialize for BitBox<O, T>
 where
 	O: BitOrder,
@@ -134,7 +147,6 @@ where
 }
 
 #[cfg(feature = "alloc")]
-#[cfg(not(tarpaulin_include))]
 impl<O, T> Serialize for BitVec<O, T>
 where
 	O: BitOrder,
@@ -147,72 +159,46 @@ where
 	}
 }
 
-impl<'de, O, T> Deserialize<'de> for BitArray<O, T>
+impl<'de, O, V> Deserialize<'de> for BitArray<O, V>
 where
 	O: BitOrder,
-	T: BitStore + BitRegister,
-	T::Mem: Deserialize<'de>,
+	V: BitView + Deserialize<'de>,
 {
 	fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
 	where D: Deserializer<'de> {
 		deserializer
-			.pipe(<T::Mem as Deserialize<'de>>::deserialize)?
-			.pipe(dvl::remove_mem::<T>)
-			.pipe(Self::new)
-			.pipe(Ok)
+			.pipe(<V as Deserialize<'de>>::deserialize)
+			.map(Self::new)
 	}
 }
 
-macro_rules! deser_array {
-	($($n:expr),+ $(,)?) => { $(
-		impl<'de, O, T> Deserialize<'de> for BitArray<O, [T; $n]>
-		where
-			O: BitOrder,
-			T: BitStore,
-			T::Mem: Deserialize<'de>
-		{
-			fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-			where D: Deserializer<'de> {
-				deserializer
-					.pipe(<[T::Mem; $n] as Deserialize<'de>>::deserialize)?
-					.pipe(|arr| unsafe { ptr::read(&arr as *const [T::Mem; $n] as *const [T; $n]) })
-					.pipe(Self::new)
-					.pipe(Ok)
-			}
-		}
-	)+ };
-}
-
-deser_array!(
-	0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
-	21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32
-);
+/** Aid for deserializing a protocol into a [`BitVec`].
 
+[`BitVec`]: crate::vec::BitVec
+**/
 #[cfg(feature = "alloc")]
 #[derive(Clone, Copy, Debug, Default)]
 struct BitVecVisitor<'de, O, T>
 where
 	O: BitOrder,
-	T: BitStore,
-	T::Mem: Deserialize<'de>,
+	T: BitStore + Deserialize<'de>,
 {
 	_lt: PhantomData<&'de ()>,
-	_bs: PhantomData<BitVec<O, T>>,
+	_bv: PhantomData<BitVec<O, T>>,
 }
 
 #[cfg(feature = "alloc")]
 impl<'de, O, T> BitVecVisitor<'de, O, T>
 where
 	O: BitOrder,
-	T: BitStore,
-	T::Mem: Deserialize<'de>,
+	T: BitStore + Deserialize<'de>,
 {
 	const THIS: Self = Self {
 		_lt: PhantomData,
-		_bs: PhantomData,
+		_bv: PhantomData,
 	};
 
-	/// Constructs a `BitVec` from deserialized components.
+	/// Constructs a [`BitVec`] from deserialized components.
 	///
 	/// # Parameters
 	///
@@ -223,46 +209,67 @@ where
 	///
 	/// # Returns
 	///
-	/// The result of assembling the deserialized components into a `BitVec`.
+	/// The result of assembling the deserialized components into a [`BitVec`].
 	/// This can fail if the `head` is invalid, or if the deserialized data
-	/// cannot be encoded into a `BitPtr`.
+	/// cannot be encoded into a `BitSpan`.
+	///
+	/// [`BitVec`]: crate::vec::BitVec
 	fn assemble<E>(
 		&self,
 		head: u8,
 		bits: usize,
-		data: Vec<T::Mem>,
+		data: Vec<T>,
 	) -> Result<<Self as Visitor<'de>>::Value, E>
 	where
 		E: de::Error,
 	{
 		//  Disable the destructor on the deserialized buffer
-		let data = ManuallyDrop::new(data);
-		//  Assemble a region pointer
-		BitPtr::new(
-			data.as_ptr() as *mut T,
-			//  Attempt to read the `head` index as a `BitIdx` bounded by the
-			//  destination type.
-			head.try_into().map_err(|_| {
-				de::Error::invalid_value(
-					Unexpected::Unsigned(head as u64),
-					&"a head-bit index less than the deserialized element \
-					  type’s bit width",
-				)
-			})?,
-			//  Ensure that the `bits` counter is not lying about the data size.
-			cmp::min(bits, data.len().saturating_mul(T::Mem::BITS as usize)),
-		)
-		//  Fail if the source cannot be encoded into a bit pointer.
-		.ok_or_else(|| {
-			de::Error::invalid_value(
-				Unexpected::Other("invalid bit-region source data"),
-				self,
-			)
-		})?
-		//  No more branches remain, only typesystem manipulation,
-		.pipe(BitPtr::to_bitslice_ptr_mut)
-		.pipe(|bp| unsafe { BitVec::from_raw_parts(bp, data.capacity()) })
-		.pipe(Ok)
+		let mut data = ManuallyDrop::new(data);
+		//  Ensure that the `bits` counter is not lying about the data size.
+		let bits = cmp::min(
+			bits,
+			data.len()
+				.saturating_mul(T::Mem::BITS as usize)
+				.saturating_sub(head as usize),
+		);
+		//  Assemble a pointer to the start bit,
+		BitPtr::try_new(data.as_mut_slice().as_mut_ptr(), head)
+			.map_err(Into::into)
+			//  Extend it into a region descriptor,
+			.and_then(|bp| bp.span(bits))
+			//  Capture the errors that can arise from the deser data,
+			.map_err(|err| match err {
+				BitSpanError::InvalidBitptr(BitPtrError::InvalidIndex(err)) => {
+					de::Error::invalid_value(
+						Unexpected::Unsigned(err.value() as u64),
+						&"a head-bit index less than the deserialized element \
+						  type’s bit width",
+					)
+				},
+				BitSpanError::TooLong(len) => de::Error::invalid_value(
+					Unexpected::Unsigned(len as u64),
+					&"a bit length that can be encoded into a `*BitSlice` \
+					  pointer",
+				),
+				BitSpanError::TooHigh(_) => unreachable!(
+					"The allocator will not produce a vector too high in the \
+					 memory space"
+				),
+				BitSpanError::InvalidBitptr(BitPtrError::InvalidAddress(
+					AddressError::Null,
+				)) => {
+					unreachable!("The allocator will not produce a null pointer")
+				},
+				BitSpanError::InvalidBitptr(BitPtrError::InvalidAddress(
+					AddressError::Misaligned(_),
+				)) => {
+					unreachable!(
+						"The allocator will not produce a misaligned buffer"
+					)
+				},
+			})
+			//  And assemble a bit-vector over the allocated span.
+			.map(|span| unsafe { BitVec::from_fields(span, data.capacity()) })
 	}
 }
 
@@ -270,18 +277,16 @@ where
 impl<'de, O, T> Visitor<'de> for BitVecVisitor<'de, O, T>
 where
 	O: BitOrder,
-	T: BitStore,
-	T::Mem: Deserialize<'de>,
+	T: BitStore + Deserialize<'de>,
 {
 	type Value = BitVec<O, T>;
 
 	fn expecting(&self, fmt: &mut Formatter) -> fmt::Result {
-		fmt.write_str("a BitSet data series")
+		fmt.write_str("a BitSeq data series")
 	}
 
 	/// Visit a sequence of anonymous data elements. These must be in the order
-	/// `u8` (head-bit index), `u64` (length counter), `[T::Mem]` (data
-	/// contents).
+	/// `u8` (head-bit index), `u64` (length counter), `[T]` (data contents).
 	fn visit_seq<V>(self, mut seq: V) -> Result<Self::Value, V::Error>
 	where V: SeqAccess<'de> {
 		let head = seq
@@ -291,19 +296,19 @@ where
 			.next_element::<u64>()?
 			.ok_or_else(|| de::Error::invalid_length(1, &self))?;
 		let data = seq
-			.next_element::<Vec<T::Mem>>()?
+			.next_element::<Vec<T>>()?
 			.ok_or_else(|| de::Error::invalid_length(2, &self))?;
 
 		self.assemble(head, bits as usize, data)
 	}
 
 	/// Visit a map of named data elements. These may be in any order, and must
-	/// be the pairs `head: u8`, `bits: u64`, and `data: [T::Mem]`.
+	/// be the pairs `head: u8`, `bits: u64`, and `data: [T]`.
 	fn visit_map<V>(self, mut map: V) -> Result<Self::Value, V::Error>
 	where V: MapAccess<'de> {
 		let mut head: Option<u8> = None;
 		let mut bits: Option<u64> = None;
-		let mut data: Option<Vec<T::Mem>> = None;
+		let mut data: Option<Vec<T>> = None;
 
 		while let Some(key) = map.next_key()? {
 			match key {
@@ -323,6 +328,11 @@ where
 					}
 				},
 				f => {
+					/* Once a key is pulled from the map, a value **must** also
+					be pulled, otherwise `serde` will fail with its own error
+					rather than this one.
+					*/
+					let _ = map.next_value::<()>();
 					return Err(de::Error::unknown_field(f, &[
 						"head", "bits", "data",
 					]));
@@ -341,10 +351,8 @@ where
 impl<'de, O, T> Deserialize<'de> for BitBox<O, T>
 where
 	O: BitOrder,
-	T: BitStore,
-	T::Mem: Deserialize<'de>,
+	T: BitStore + Deserialize<'de>,
 {
-	#[inline]
 	fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
 	where D: Deserializer<'de> {
 		deserializer
@@ -357,13 +365,12 @@ where
 impl<'de, O, T> Deserialize<'de> for BitVec<O, T>
 where
 	O: BitOrder,
-	T: BitStore,
-	T::Mem: Deserialize<'de>,
+	T: BitStore + Deserialize<'de>,
 {
 	fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
 	where D: Deserializer<'de> {
 		deserializer.deserialize_struct(
-			"BitSet",
+			"BitSeq",
 			&["head", "bits", "data"],
 			BitVecVisitor::THIS,
 		)
@@ -374,6 +381,8 @@ where
 mod tests {
 	use crate::prelude::*;
 
+	use serde::Deserialize;
+
 	use serde_test::{
 		assert_ser_tokens,
 		Token,
@@ -388,7 +397,7 @@ mod tests {
 	macro_rules! bvtok {
 		( s $elts:expr, $head:expr, $bits:expr, $ty:ident $( , $data:expr )* ) => {
 			&[
-				Token::Struct { name: "BitSet", len: 3, },
+				Token::Struct { name: "BitSeq", len: 3, },
 				Token::Str("head"), Token::U8( $head ),
 				Token::Str("bits"), Token::U64( $bits ),
 				Token::Str("data"), Token::Seq { len: Some( $elts ) },
@@ -399,7 +408,7 @@ mod tests {
 		};
 		( d $elts:expr, $head:expr, $bits:expr, $ty:ident $( , $data:expr )* ) => {
 			&[
-				Token::Struct { name: "BitSet", len: 3, },
+				Token::Struct { name: "BitSeq", len: 3, },
 				Token::BorrowedStr("head"), Token::U8( $head ),
 				Token::BorrowedStr("bits"), Token::U64( $bits ),
 				Token::BorrowedStr("data"), Token::Seq { len: Some( $elts ) },
@@ -436,7 +445,6 @@ mod tests {
 	}
 
 	#[test]
-	#[cfg(feature = "alloc")]
 	fn wide() {
 		let src: &[u8] = &[0, !0];
 		let bs = src.view_bits::<LocalBits>();
@@ -447,11 +455,22 @@ mod tests {
 	#[cfg(feature = "alloc")]
 	fn deser() {
 		let bv = bitvec![Msb0, u8; 0, 1, 1, 0, 1, 0];
+		let bb = bv.clone().into_boxed_bitslice();
 		assert_de_tokens(&bv, bvtok![d 1, 0, 6, U8, 0b0110_1000]);
 		//  test that the bits outside the bits domain don't matter in deser
 		assert_de_tokens(&bv, bvtok![d 1, 0, 6, U8, 0b0110_1001]);
-		assert_de_tokens(&bv, bvtok![d 1, 0, 6, U8, 0b0110_1010]);
-		assert_de_tokens(&bv, bvtok![d 1, 0, 6, U8, 0b0110_1011]);
+		assert_de_tokens(&bb, bvtok![d 1, 0, 6, U8, 0b0110_1010]);
+		assert_de_tokens(&bb, bvtok![d 1, 0, 6, U8, 0b0110_1011]);
+	}
+
+	#[test]
+	#[cfg(feature = "alloc")]
+	fn ser() {
+		let bv = bitvec![Msb0, u8; 0, 1, 1, 0, 1, 0];
+		let bb = bv.clone().into_boxed_bitslice();
+
+		assert_ser_tokens(&bv, bvtok![s 1, 0, 6, U8, 0b0110_1000]);
+		assert_ser_tokens(&bb, bvtok![s 1, 0, 6, U8, 0b0110_1000]);
 	}
 
 	#[test]
@@ -462,5 +481,97 @@ mod tests {
 			"invalid value: integer `9`, expected a head-bit index less than \
 			 the deserialized element type’s bit width",
 		);
+
+		for field in &["head", "bits"] {
+			assert_de_tokens_error::<BitVec<Msb0, u8>>(
+				&[
+					Token::Struct {
+						name: "BitSeq",
+						len: 2,
+					},
+					Token::BorrowedStr(field),
+					Token::U8(0),
+					Token::BorrowedStr(field),
+					Token::U8(1),
+					Token::StructEnd,
+				],
+				&format!("duplicate field `{}`", field),
+			);
+		}
+
+		assert_de_tokens_error::<BitVec<Msb0, u8>>(
+			&[
+				Token::Struct {
+					name: "BitSeq",
+					len: 2,
+				},
+				Token::BorrowedStr("data"),
+				Token::Seq { len: Some(1) },
+				Token::U8(2),
+				Token::SeqEnd,
+				Token::BorrowedStr("data"),
+				Token::Seq { len: Some(1) },
+				Token::U8(3),
+				Token::SeqEnd,
+				Token::StructEnd,
+			],
+			"duplicate field `data`",
+		);
+
+		assert_de_tokens_error::<BitVec<Msb0, u8>>(
+			&[
+				Token::Struct {
+					name: "BitSeq",
+					len: 1,
+				},
+				Token::BorrowedStr("garbage"),
+				Token::BorrowedStr("field"),
+				Token::StructEnd,
+			],
+			"unknown field `garbage`, expected one of `head`, `bits`, `data`",
+		);
+	}
+
+	#[test]
+	fn deser_seq() {
+		let bv = bitvec![Msb0, u8; 0, 1];
+		assert_de_tokens::<BitVec<Msb0, u8>>(&bv, &[
+			Token::Seq { len: Some(3) },
+			Token::U8(0),
+			Token::U64(2),
+			Token::Seq { len: Some(1) },
+			Token::U8(66),
+			Token::SeqEnd,
+			Token::SeqEnd,
+		]);
+
+		assert_de_tokens_error::<BitVec<Msb0, u8>>(
+			&[Token::Seq { len: Some(0) }, Token::SeqEnd],
+			"invalid length 0, expected a BitSeq data series",
+		);
+
+		assert_de_tokens_error::<BitVec<Msb0, u8>>(
+			&[Token::Seq { len: Some(1) }, Token::U8(0), Token::SeqEnd],
+			"invalid length 1, expected a BitSeq data series",
+		);
+
+		assert_de_tokens_error::<BitVec<Msb0, u8>>(
+			&[
+				Token::Seq { len: Some(2) },
+				Token::U8(0),
+				Token::U64(2),
+				Token::SeqEnd,
+			],
+			"invalid length 2, expected a BitSeq data series",
+		);
+	}
+
+	#[test]
+	fn trait_impls() {
+		const _: fn() = || {
+			fn assert_impl_all<'de, T: Deserialize<'de>>() {
+			}
+			assert_impl_all::<BitArray<LocalBits, [usize; 32]>>();
+		};
 	}
 }
